/*
 *  Copyright (C) 2007  Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <villa.h>
#include <stdlib.h>
#include "avaliacao.h"

gchar *regex_str;
gsize regex_len;
GRegex *regex;

gboolean
regex_eval (const GMatchInfo *info, GString *result, gpointer data)
{
  char *smap;
  int slen;
  CBMAP *map;
  gchar *match;
  const char *val;
  match = g_match_info_fetch (info, 1);
  smap = vlcurval (db, &slen);
  map = cbmapload (smap, slen);
  val = cbmapget (map, match + 1, -1, NULL);
  g_string_append (result, val);
  g_free (smap);
  g_free (match);
  cbmapclose (map);
  return FALSE;
}

char *
regex_create_message ()
{
  GError *error;
  char *message;
  error = NULL;
  message = g_regex_replace_eval (regex, regex_str, regex_len,
				  0, 0, regex_eval, NULL, &error);
  return message;
}

void
regex_init ()
{
  GIOChannel *channel;
  GError *error;
  GIOStatus st;
  error = NULL;
  regex_str = NULL;
  channel = g_io_channel_new_file ("message", "r", &error);
  if (channel == NULL)
    {
      g_critical ("Could not open file: %s", error->message);
      g_error_free (error);
      abort ();
    }
  st = g_io_channel_read_to_end (channel, &regex_str, &regex_len, &error);
  if (st != G_IO_STATUS_NORMAL)
    {
      g_critical ("Could not read file: %s", error->message);
      g_error_free (error);
      abort ();
    }
  g_io_channel_close (channel);
  regex = g_regex_new ("(?<!\\\\)(\\\\[[:alpha:]][[:alnum:]]*)", 0, 0, &error);
  if (regex == NULL)
    {
      g_critical ("Error creating regex: %s", error->message);
      g_error_free (error);
      g_free (regex_str);
      abort ();
    }
}
